Package: AllelicImbalance
Type: Package
Title: Investigates Allele Specific Expression
Version: 1.51.0
Date: 2021-11-17
Encoding: UTF-8
Author: Jesper R Gadin, Lasse Folkersen
Maintainer: Jesper R Gadin <j.r.gadin@gmail.com>
Description: Provides a framework for allelic specific expression
    investigation using RNA-seq data.
License: GPL-3
URL: https://github.com/pappewaio/AllelicImbalance
BugReports: https://github.com/pappewaio/AllelicImbalance/issues
Suggests: testthat, org.Hs.eg.db, TxDb.Hsapiens.UCSC.hg19.knownGene,
        SNPlocs.Hsapiens.dbSNP144.GRCh37, BiocStyle, knitr, rmarkdown
Depends: R (>= 4.0.0), grid, GenomicRanges (>= 1.31.8),
        SummarizedExperiment (>= 0.2.0), GenomicAlignments (>= 1.15.6)
Imports: methods, BiocGenerics, AnnotationDbi, BSgenome (>= 1.47.3),
        VariantAnnotation (>= 1.25.11), Biostrings (>= 2.47.6),
        S4Vectors (>= 0.17.25), IRanges (>= 2.13.12), Rsamtools (>=
        1.99.3), GenomicFeatures (>= 1.31.3), Gviz, lattice,
        latticeExtra, gridExtra, seqinr, GenomeInfoDb, nlme
LazyData: TRUE
biocViews: Genetics, Infrastructure, Sequencing
VignetteBuilder: knitr
Collate: 'AllelicImbalance-package.R' 'initialize-methods.R'
        'ASEset-class.R' 'DetectedAI-class.R' 'GlobalAnalysis-class.R'
        'barplot-methods.R' 'locationplot-methods.R'
        'GvizTrack-methods.R' 'LinkVariantAlmlof-class.R'
        'RegionSummary-class.R' 'RiskVariant-class.R'
        'auxillary-functions-annotation.R'
        'auxillary-functions-visuals.R'
        'auxillary-methods-annotation.R'
        'auxillary-methods-summaries.R' 'auxillary-methods.R'
        'chisq.test-methods.R' 'binom.test-methods.R'
        'boxplot-methods.R' 'deprecations.R' 'detect-methods.R'
        'filter-methods.R' 'histplot-methods.R' 'inference-methods.R'
        'linkage-methods.R' 'list-methods.R' 'mapbias-methods.R'
        'plot-methods.R' 'show-methods.R' 'simulation-methods.R'
        'summary-methods.R' 'utils.R'
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/AllelicImbalance
git_branch: devel
git_last_commit: 718ddd4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:14:27 UTC; biocbuild
