##'@include ASEset-class.R
#NULL
#
##' genotype filter methods 
##' 
##' useful genotype filters
##' 
##' These filters are called upon ASEset objects 
##' 
##' @name ReferenceBias-show
##' @rdname ReferenceBias-show
##' @aliases ReferenceBias-show ReferenceBias-show,ReferenceBias-method show,ReferenceBias-method
##' @docType methods
##' @param object ReferenceBias object
##' @author Jesper R. Gadin, Lasse Folkersen
##' @keywords show
##' @examples
##' 
##' #load example data
##' cat("there is no example data yet for this method")
##' 
##' @importFrom methods show
##' @exportMethod show
#
#setMethod("show","ReferenceBias",
#	function(object)
#		{
#
#		#Header
#		cat("Class:","ReferenceBias\n")
#		cat("Contains: RefFraction array\n")
#		}
#)
#
#
