% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference-methods.R
\name{inferGenotypes}
\alias{inferGenotypes}
\alias{inferGenotypes,ASEset-method}
\title{infererence of genotypes from ASEset count data}
\usage{
inferGenotypes(
  x,
  strand = "*",
  return.class = "matrix",
  return.allele.allowed = "bi",
  threshold.frequency = 0,
  threshold.count.sample = 1
)
}
\arguments{
\item{x}{ASEset}

\item{strand}{strand to infer from}

\item{return.class}{'matrix' or 'vector'}

\item{return.allele.allowed}{vector with 'bi' 'tri' or 'quad'.
'uni' Always gets returned}

\item{threshold.frequency}{least fraction to classify (see details)}

\item{threshold.count.sample}{least amount of counts to try to infer allele}
}
\description{
inference of genotypes
}
\details{
Oftern necessary information to link AI to SNPs outside coding region
}
\examples{

data(ASEset)
g <- inferGenotypes(ASEset)

}
\author{
Jesper R. Gadin
}
\keyword{infer}
